PacketSequenceCount

Struct PacketSequenceCount 

pub struct PacketSequenceCount(/* private fields */);
Packet sequence count

The packet sequence count is unique per APID and denotes the sequential binary count of each Space Packet (generated per APID). For telecommands (i.e., with packet type 1) this may also be a “packet name” that identifies the telecommand packet within its communications session.

Implementations

Source

impl PacketSequenceCount

pub const fn new() -> Self

The packet sequence count is initialized to zero by default.

pub const fn increment(&mut self)

A good default behaviour is for the packet sequence count to increment by one every time a new packet is sent. This method permits a simple wrapping increment to be performed, to make this easier.