PacketVersionNumber

Struct PacketVersionNumber 

pub struct PacketVersionNumber(/* private fields */);
The packet version number represents the version of the Space Packet protocol that is used. In the version presently implemented, this is defined to be zeroes.

Implementations

Source

impl PacketVersionNumber

pub fn is_supported(&self) -> bool

The Space Packet protocol version presently implemented in this crate is based on issue 2 of the CCSDS SPP blue book, which encompasses only the Version 1 CCSDS Packet, indicated by a version number of 0. Other packet structures may be added in the future.

pub fn version1_ccsds_packet() -> Self

Returns the packet version number corresponding with the Version 1 CCSDS Packet.