SpacePacketPrimaryHeader

Struct SpacePacketPrimaryHeader 

pub struct SpacePacketPrimaryHeader { /* private fields */ }
Space packet primary header

Representation of only the fixed-size primary header part of a space packet. Used to construct generic space packets, but mostly useful in permitting composition of derived packet types, like PUS packets; otherwise, the dynamically-sized data field member would get in the way of including the primary header directly in derived packets.

Implementations

Source

impl SpacePacketPrimaryHeader

pub const fn primary_header_size() -> usize

Returns the size of a Space Packet primary header, in bytes. In the version that is presently implemented, that is always 6 bytes.

pub const fn packet_version(&self) -> PacketVersionNumber

Since the Space Packet protocol may technically support alternative packet structures in future versions, the 3-bit packet version field may not actually contain a “correct” value.

pub fn initialize_packet_version(&mut self)

Initializes the packet version to the proper value. Must be a fixed value, so this function takes no arguments.

pub const fn packet_type(&self) -> PacketType

The packet type denotes whether a packet is a telecommand (request) or telemetry (report) packet. Note that the exact definition of telecommand and telemetry may differ per system, and indeed the “correct” value here may differ per project.

pub fn set_packet_type(&mut self, packet_type: PacketType)

Sets the packet type to the given value.

pub const fn secondary_header_flag(&self) -> SecondaryHeaderFlag

Denotes whether the packet contains a secondary header. If no user field is present, the secondary header is mandatory (presumably, to ensure that some data is always transferred, considering the Space Packet header itself contains no meaningful data).

pub fn set_secondary_header_flag( &mut self, secondary_header_flag: SecondaryHeaderFlag, )

Updates the value of the secondary header flag with the provided value.

pub const fn apid(&self) -> Apid

Returns the application process ID stored in the packet. The actual meaning of this APID field may differ per implementation: technically, it only represents “some” data path. In practice, it will often be a identifier for a data channel, the packet source, or the packet destination.

pub fn set_apid(&mut self, apid: Apid)

Sets the APID used to route the packet to the given value.

pub const fn sequence_flag(&self) -> SequenceFlag

Sequence flags may be used to indicate that the data contained in a packet is only part of a larger set of application data.

pub fn set_sequence_flag(&mut self, sequence_flag: SequenceFlag)

Sets the sequence flag to the provided value.

pub const fn packet_sequence_count(&self) -> PacketSequenceCount

The packet sequence count is unique per APID and denotes the sequential binary count of each Space Packet (generated per APID). For telecommands (i.e., with packet type 1) this may also be a “packet name” that identifies the telecommand packet within its communications session.

pub fn set_packet_sequence_count(&mut self, sequence_count: PacketSequenceCount)

Sets the packet sequence count to the provided value. This value must be provided by an external counter and is not provided at a Space Packet type level because it might differ between packet streams.

pub const fn packet_data_length(&self) -> usize

The packet data length field represents the length of the associated packet data field. However, it is not stored directly: rather, the “length count” is stored, which is the packet data length minus one.

pub fn set_packet_data_length( &mut self, packet_data_length: usize, ) -> Result<(), InvalidPacketDataLength>

Sets the packet data length field to the provided value. Note that the given value is not stored directly, but rather decremented by one first. Accordingly, and as per the CCSDS Space Packet Protocol standard, packet data lengths of 0 are not allowed.

Errors

Shall return an error if an empty packet data length is requested.