PacketAssemblyError

Enum PacketAssemblyError 

#[non_exhaustive]
pub enum PacketAssemblyError { BufferTooSmall { buffer_length: usize, packet_length: usize, }, EmptyDataFieldRequested, }
Representation of the set of errors that may be encountered while constructing a Space Packet. Marked as non-exhaustive to permit extension with additional semantic errors in the future without breaking API.

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


BufferTooSmall

Returned when the underlying buffer does not have sufficient bytes to contain a given space packet.

Fields

buffer_length: usize
packet_length: usize


EmptyDataFieldRequested

As per the CCSDS standard, Space Packets shall have at least one byte in their data field. Hence, requests for an empty data field must be rejected.