pub trait PacketReception {
// Required method
fn receive(&mut self) -> Option<&SpacePacket>;
}Expand description
The PacketReception trait describes the “Packet Reception” function from the CCSDS 133.0-B-2
Space Packet Protocol recommended standard. It concerns the ability to receive new Space
Packets from some underlying subnetwork layer.
Required Methods
Sourcefn receive(&mut self) -> Option<&SpacePacket>
fn receive(&mut self) -> Option<&SpacePacket>
Polls the message bus to see if new Space Packets have been received for a given APID. If so, returns a reference to it. Need not perform any checking, such as data loss checks: that may be done by the receiving party.
After reception, the Space Packet shall be removed from the packet receptor: on future
polls (for the same self), it shall no longer be returned.